import pandas as pd
from google.cloud import bigquery
import sroka.config.config as config
from google.api_core.exceptions import Forbidden, NotFound, BadRequest


KEY_FILE = config.get_file_path('google_bigquery')


def query_bigquery(input_query, filename=None):

    if filename:
        if type(filename) != str:
            print('filename needs to be a string')
            return None

        if type(input_query) != str:
            print('input_query needs to be a string')
            return None

        try:
            f = open(filename, 'w')
            f.close()
        except FileNotFoundError:
            print('file cannot be saved in selected directory')
            return None

    else:
        if type(input_query) != str:
            print('input_query needs to be a string')
            return pd.DataFrame([])

    client = bigquery.Client.from_service_account_json(
        KEY_FILE)

    query_job = client.query(input_query)

    try:
        df = query_job.result().to_dataframe()

    except (NotFound, BadRequest) as error:
        print(error)
        if filename:
            return None
        return pd.DataFrame([])

    if filename:
        df.to_csv(filename)
        print('saved to ' + filename)
        return None
    else:
        return df


def done_bigquery(job_id, filename=None):

    if filename:
        if type(filename) != str:
            print('filename needs to be a string')
            return None

        if type(job_id) != str:
            print('input_query needs to be a string')
            return None

        try:
            f = open(filename, 'w')
            f.close()
        except FileNotFoundError:
            print('file cannot be saved in selected directory')
            return None

    else:
        if type(job_id) != str:
            print('input_query needs to be a string')
            return pd.DataFrame([])

    client = bigquery.Client.from_service_account_json(
        KEY_FILE)
    try:
        query_job = client.get_job(job_id=job_id)
    except (BadRequest, NotFound) as error:
        print(error)
        if filename:
            return None
        return pd.DataFrame([])
    try:
        df = query_job.result().to_dataframe()
    except (Forbidden, NotFound) as error:
        print(error)
        if filename:
            return None
        return pd.DataFrame([])
    if filename:
        df.to_csv(filename)
        print('saved to ' + filename)
        return None
    else:
        return df



import gzip
import tempfile
from configparser import NoOptionError

import pandas as pd
from googleads import ad_manager, errors

import sroka.config.config as config

# import variable_validators as validator

KEY_FILE = config.get_file_path('google_ad_manager')

# GAM API information.
try:
    APPLICATION_NAME = config.get_value('google_ad_manager', 'application_name')
except (KeyError, NoOptionError):
    APPLICATION_NAME = 'Application name'


def dict_type_checker(dict_argument, argument_name, mandatory=True):
    if mandatory:
        if type(dict_argument) != dict:
            print(f"""{argument_name} needs to be a dict""")
            return "Incorrect type"
    else:
        if dict_argument and type(dict_argument) != dict:
            print(f"""{argument_name} needs to be a dict""")
            return "Incorrect type"


def int_type_checker(int_argument, argument_name, mandatory=True):
    if mandatory:
        if type(int_argument) != int:
            print(f"""{argument_name} needs to be an integer""")
            return "Incorrect type"
    else:
        if int_argument and type(int_argument) != int:
            print(f"""{argument_name} needs to be an integer""")
            return "Incorrect type"


def list_type_checker(list_argument, argument_name, mandatory=True):
    if mandatory:
        if type(list_argument) != list:
            print(f"""{argument_name} needs to be a list""")
            return "Incorrect type"
    else:
        if list_argument and type(list_argument) != list:
            print(f"""{argument_name} needs to be a list""")
            return "Incorrect type"


def str_type_checker(str_argument, argument_name, mandatory=True):
    if mandatory:
        if type(str_argument) != str:
            print(f"""{argument_name} needs to be a string""")
            return "Incorrect type"
    else:
        if str_argument and type(str_argument) != str:
            print(f"""{argument_name} needs to be a string""")
            return "Incorrect type"


def init_gam_connection(network_code=None):
    if not network_code:
        try:
            network_code = config.get_value('google_ad_manager', 'network_code')
        except (KeyError, NoOptionError):
            print('No network code was provided')
            return pd.DataFrame([])
    yaml_string = "ad_manager: " + "\n" + \
                  "  application_name: " + APPLICATION_NAME + "\n" + \
                  "  network_code: " + str(network_code) + "\n" + \
                  "  path_to_private_key_file: " + KEY_FILE + "\n"

    # Initialize the GAM client.
    gam_client = ad_manager.AdManagerClient.LoadFromString(yaml_string)
    return gam_client


def get_data_from_admanager(query, dimensions, columns, start_date, end_date, custom_field_id=None,
                            dimension_attributes=None, network_code=None):
    if not custom_field_id:
        custom_field_id = []

    if not dimension_attributes:
        dimension_attributes = []

    list_of_types = [str_type_checker(query, "query"),
                     list_type_checker(dimensions, "dimensions"),
                     list_type_checker(columns, "columns"),
                     dict_type_checker(start_date, "start_date"),
                     dict_type_checker(end_date, "end_date"),
                     list_type_checker(custom_field_id, "custom_field_id", False),
                     list_type_checker(dimension_attributes, "dimmension_attributes", False),
                     int_type_checker(network_code, "network_code", False)]

    if "Incorrect type" in list_of_types:
        return

    gam_client = init_gam_connection(network_code)

    # Create statement object to filter for an order.

    filter_statement = {'query': query}

    # Create report job.
    report_job = {
        'reportQuery': {
            'dimensions': dimensions,
            'statement': filter_statement,
            'columns': columns,
            'customFieldIds': custom_field_id,
            'dimensionAttributes': dimension_attributes,
            'dateRangeType': 'CUSTOM_DATE',
            'startDate': start_date,
            'endDate': end_date,
            'adUnitView': "FLAT"
        }
    }

    report_downloader = gam_client.GetDataDownloader()

    try:
        # Run the report and wait for it to finish.
        report_job_id = report_downloader.WaitForReport(report_job)
    except errors.GoogleAdsServerFault as e:
        if 'AuthenticationError.NETWORK_NOT_FOUND' in str(e):
            print('Provided network code was not found.')
        elif 'AuthenticationError.NETWORK_CODE_REQUIRED' in str(e):
            print('Default value of network code is missing from ', config.default_config_filepath)
        else:
            print('Failed to generate report. Error was: {}'.format(e))
        return

    except errors.AdManagerReportError as e:
        print('Failed to generate report. Error was: {}'.format(e))
        return

    export_format = 'CSV_DUMP'

    report_file = tempfile.NamedTemporaryFile(suffix='.csv.gz', delete=False)

    # Download report data.
    report_downloader.DownloadReportToFile(
        report_job_id, export_format, report_file, use_gzip_compression=True)

    report_file.close()

    # Display results.
    print('Report job with id {} downloaded to:\n{}'.format(
        report_job_id, report_file.name))
    with gzip.open(report_file.name) as file:
        data = pd.read_csv(file)
    return data


def get_users_from_admanager(query, dimensions, network_code=None):
    list_of_types = [str_type_checker(query, "query"),
                     list_type_checker(dimensions, "dimensions"),
                     int_type_checker(network_code, "network_code", False)]

    if "Incorrect type" in list_of_types:
        return

    user_df = pd.DataFrame()

    dimensions_df = pd.DataFrame()

    # .Where() handles filtering with admanager method, while statement_query handles queries with WHERE clause

    statement_query = query.upper().replace("WHERE ", "")

    statement = ad_manager.StatementBuilder().Where(statement_query)

    gam_client = init_gam_connection(network_code)

    user_service = gam_client.GetService('UserService')

    try:
        while True:
            response = user_service.getUsersByStatement(statement.ToStatement())
            if 'results' in response and len(response['results']):
                for user in response['results']:

                    for dimension in dimensions:
                        try:
                            dimension_value = [user[dimension]]
                            dimensions_df[dimension] = dimension_value
                        except KeyError as e:
                            print('Failed to generate user list. Incorrect dimension: {}'.format(e))
                            return

                    user_df = user_df.append(dimensions_df, sort=False)
                statement.offset += statement.limit
            else:
                break
        return user_df

    except errors.GoogleAdsServerFault as e:
        if 'AuthenticationError.NETWORK_NOT_FOUND' in str(e):
            print('Provided network code was not found.')
        elif 'AuthenticationError.NETWORK_CODE_REQUIRED' in str(e):
            print('Default value of network code is missing from ', config.default_config_filepath)
        else:
            print('Failed to generate user list. Error was: {}'.format(e))
        return

    except errors.AdManagerReportError as e:
        print('Failed to generate user list. Error was: {}'.format(e))
        return


def get_companies_from_admanager(query, dimensions, network_code=None):
    list_of_types = [str_type_checker(query, "query"),
                     list_type_checker(dimensions, "dimensions"),
                     int_type_checker(network_code, "network_code", False)]

    if "Incorrect type" in list_of_types:
        return

    company_df = pd.DataFrame()

    dimensions_df = pd.DataFrame()

    # .Where() handles filtering with admanager method, while statement_query handles queries with WHERE clause

    statement_query = query.upper().replace("WHERE ", "")

    statement = ad_manager.StatementBuilder().Where(statement_query)

    gam_client = init_gam_connection(network_code)

    company_service = gam_client.GetService('CompanyService')

    try:
        while True:
            response = company_service.getCompaniesByStatement(statement.ToStatement())
            if 'results' in response and len(response['results']):
                for company in response['results']:

                    for dimension in dimensions:
                        try:
                            dimension_value = [company[dimension]]
                            dimensions_df[dimension] = dimension_value
                        except KeyError as e:
                            print('Failed to generate company list. Incorrect dimension: {}'.format(e))
                            return

                    company_df = company_df.append(dimensions_df, sort=False)
                statement.offset += statement.limit
            else:
                break
        return company_df

    except errors.GoogleAdsServerFault as e:
        if 'AuthenticationError.NETWORK_NOT_FOUND' in str(e):
            print('Provided network code was not found.')
        elif 'AuthenticationError.NETWORK_CODE_REQUIRED' in str(e):
            print('Default value of network code is missing from ', config.default_config_filepath)
        else:
            print('Failed to generate company list. Error was: {}'.format(e))
        return

    except errors.AdManagerReportError as e:
        print('Failed to generate company list. Error was: {}'.format(e))
        return



from __future__ import print_function

import os

import numpy as np
import pandas as pd
from googleapiclient.discovery import build
from googleapiclient.errors import HttpError

import sroka.config.config as config


def google_drive_sheets_read(sheetname_id, sheet_range, first_row_columns=False):
    """
    Function to read from google sheets
    """
    # Authenticate and construct service.
    scope = 'https://www.googleapis.com/auth/spreadsheets.readonly'
    key_file_location = config.get_file_path('google_drive')
    authorized_user_file = os.path.expanduser('~/.cache/token_read.json')

    credentials = config.set_google_credentials(authorized_user_file,
                                                key_file_location,
                                                scope)

    service = build('sheets', 'v4', credentials=credentials)

    sheet = service.spreadsheets()
    try:
        result = sheet.values().get(spreadsheetId=sheetname_id,
                                    range=sheet_range).execute()
    except HttpError as err:
        print("HTTP error occurred. Error:")
        print(err)
        return pd.DataFrame([])

    values = result.get('values', [])
    df = pd.DataFrame(values)

    if first_row_columns:
        df.columns = df.loc[0, :].values
        df.drop(0, inplace=True)
        df.reset_index(drop=True, inplace=True)
    return df


def google_drive_sheets_create(name):
    """
    Function to create a google sheet file
    """
    scope = 'https://www.googleapis.com/auth/drive'
    key_file_location = config.get_file_path('google_drive')
    authorized_user_file = os.path.expanduser('~/.cache/token_create.json')

    credentials = config.set_google_credentials(authorized_user_file,
                                                key_file_location,
                                                scope)

    service = build('sheets', 'v4', credentials=credentials)
    spreadsheet = {
        'properties': {
            'title': name
        }
    }
    try:
        spreadsheet = service.spreadsheets().create(body=spreadsheet,
                                                    fields='spreadsheetId').execute()
    except HttpError as err:
        print("HTTP error occurred. Error:")
        print(err)
        return ''

    return spreadsheet.get('spreadsheetId')


def google_drive_sheets_add_tab(spreadsheet_id, name):
    """
    Function to add new tab to existing spreadsheet
    """
    scope = 'https://www.googleapis.com/auth/drive'
    key_file_location = config.get_file_path('google_drive')
    authorized_user_file = os.path.expanduser('~/.cache/token_create.json')

    credentials = config.set_google_credentials(authorized_user_file,
                                                key_file_location,
                                                scope)

    service = build('sheets', 'v4', credentials=credentials)

    data = [{
        'addSheet': {
            'properties': {'title': name}
        }
    }]
    try:
        spreadsheet = service.spreadsheets().batchUpdate(spreadsheetId=spreadsheet_id,
                                                         body={'requests': data},
                                                         fields='replies/addSheet').execute()
    except HttpError as err:
        print("HTTP error occurred. Error:")
        print(err)
        return spreadsheet_id

    return spreadsheet.get('spreadsheetId')


def google_drive_sheets_write(data, spreadsheet_id, sheet_range='Sheet1!A1',
                              with_columns=True, with_index=False):
    """
    Function to write to existing google sheet
    """
    scope = 'https://www.googleapis.com/auth/drive'
    key_file_location = config.get_file_path('google_drive')
    authorized_user_file = os.path.expanduser('~/.cache/token_create.json')

    credentials = config.set_google_credentials(authorized_user_file,
                                                key_file_location,
                                                scope)

    service = build('sheets', 'v4', credentials=credentials)

    values = data.values.tolist()
    columns = list(data.columns)

    if with_index:
        if values:
            values = np.append(np.array([list(data.index)]).T, values, axis=1).tolist()
        columns = ['index'] + list(data.columns)
    if with_columns:
        if values:
            values = np.append([columns], values, axis=0).tolist()
        else:
            values = [columns]

    body = {
        'values': values
    }
    try:
        service.spreadsheets().values().update(
            spreadsheetId=spreadsheet_id, range=sheet_range,
            body=body, valueInputOption='RAW').execute()
    except HttpError as err:
        print("HTTP error occurred. Error:")
        print(err)
        return None

    print('Successfully uploaded to google sheets: https://docs.google.com/spreadsheets/d/' + spreadsheet_id)
    return None


def google_drive_sheets_upload(data, name,
                               with_columns=True, with_index=False):
    """
    Function to upload data to google sheets
    """
    spreadsheet_id = google_drive_sheets_create(name)
    google_drive_sheets_write(data, spreadsheet_id, sheet_range='Sheet1!A1',
                              with_columns=with_columns, with_index=with_index)
    return spreadsheet_id



from configparser import NoOptionError
from urllib.parse import urlparse

import boto3
from botocore.exceptions import ClientError, EndpointConnectionError

import sroka.config.config as config
from sroka.api.athena.athena_api_helpers import poll_status, download_file, return_on_exception, \
    input_check


def query_athena(query, filename=None):

    if not input_check(query, [str]):
        return return_on_exception(filename)

    if not input_check(filename, [str, type(None)]):
        return return_on_exception(filename)

    if filename == '':
        print('Filename cannot be empty')
        return return_on_exception(filename)

    try:
        s3_bucket = config.get_value('aws', 's3bucket_name')
        key_id = config.get_value('aws', 'aws_access_key_id')
        access_key = config.get_value('aws', 'aws_secret_access_key')
        region = config.get_value('aws', 'aws_region')
    except (KeyError, NoOptionError) as e:
        print('No credentials were provided. Error message:')
        print(e)
        return return_on_exception(filename)

    session = boto3.Session(
        aws_access_key_id=key_id,
        aws_secret_access_key=access_key
    )

    athena = session.client('athena',
                            region_name=region)
    s3 = session.resource('s3')
    if not s3_bucket.startswith('s3://'):
        output_s3_bucket = 's3://' + s3_bucket
    else:
        output_s3_bucket = s3_bucket
        s3_bucket = s3_bucket.replace('s3://', '')
    try:
        result = athena.start_query_execution(
            QueryString=query,
            ResultConfiguration={
                'OutputLocation': output_s3_bucket,
            }
        )
    except ClientError as e:
        if e.response['Error']['Code'] == 'InvalidRequestException':
            print("Please check your query. Error message:")
        else:
            print('Please check your credentials including s3_bucket in config.ini file. Error message:')
        print(e)
        return return_on_exception(filename)

    except EndpointConnectionError as e:
        print('Please check your credentials including aws_region in config.ini file and Internet connection.',
              'Error message:')
        print(e)
        return return_on_exception(filename)

    query_id = result['QueryExecutionId']
    result = poll_status(athena, query_id)
    if result is None:
        return return_on_exception(filename)

    elif result['QueryExecution']['Status']['State'] == 'SUCCEEDED':
        s3_key = query_id + '.csv'
        return download_file(s3, s3_bucket, s3_key, filename)
    else:
        print('Query did not succeed. Reason:')
        print(result['QueryExecution']['Status']['StateChangeReason'])
        return return_on_exception(filename)


def done_athena(query_id, filename=None):

    if not input_check(query_id, [str]):
        return return_on_exception(filename)

    if not input_check(filename, [str, type(None)]):
        return return_on_exception(filename)

    try:
        s3_bucket = config.get_value('aws', 's3bucket_name')
        key_id = config.get_value('aws', 'aws_access_key_id')
        access_key = config.get_value('aws', 'aws_secret_access_key')
        region = config.get_value('aws', 'aws_region')
    except (KeyError, NoOptionError) as e:
        print('All or part of credentials were not provided. Please verify config.ini file. Error message:')
        print(e)
        return return_on_exception(filename)

    if s3_bucket.startswith('s3://'):
        s3_bucket = s3_bucket.replace('s3://', '')

    session = boto3.Session(
        aws_access_key_id=key_id,
        aws_secret_access_key=access_key
    )

    s3 = session.resource('s3')
    athena = session.client('athena',
                            region_name=region)
    result = poll_status(athena, query_id)
    if result is None:
        return return_on_exception(filename)
    if result['QueryExecution']['Status']['State'] == 'SUCCEEDED':
        s3_key = urlparse(result['QueryExecution']['ResultConfiguration']['OutputLocation']).path[1:]
        return download_file(s3, s3_bucket, s3_key, filename)
    else:
        print('Query did not succeed. Reason:')
        print(result['QueryExecution']['Status']['StateChangeReason'])
        return return_on_exception(filename)



import re
import warnings
from io import BytesIO, StringIO

import boto3
import pandas as pd
import numpy as np
import pyarrow.parquet as pq
from botocore.exceptions import ClientError, ParamValidationError
from pandas.errors import EmptyDataError

import sroka.config.config as config

warnings.filterwarnings('ignore')


def _download_data(key_prefix, s3, bucket_name, prefix, sep, skip_empty_files=True,
                   first_row_columns=True):
    if first_row_columns:
        header_setting = 'infer'
    else:
        header_setting = None
    df_list = []
    if prefix is False:
        file = s3.Object(bucket_name, key_prefix)
        try:
            data = StringIO(str(file.get()['Body'].read(), 'utf-8'))
        except UnicodeDecodeError:
            data = BytesIO(file.get()['Body'].read())
        except ClientError:
            print('File not found on s3')
            return pd.DataFrame([])
        try:
            df_list.append(pd.read_csv(data, error_bad_lines=False, warn_bad_lines=False, sep=sep,
                                       header=header_setting))
        except UnicodeDecodeError:
            df_list.append(pq.read_pandas(data).to_pandas())
        except EmptyDataError:
            print('File is empty')
            return pd.DataFrame([])

    else:
        bucket = s3.Bucket(bucket_name)

        try:
            for file in bucket.objects.filter(Prefix=key_prefix):
                if 'SUCCESS' not in file.key:
                    tmp = StringIO(str(file.get()['Body'].read(), 'utf-8'))
                    try:
                        data = pd.read_csv(tmp, error_bad_lines=False, warn_bad_lines=False, sep=sep,
                                           header=header_setting)
                        df_list.append(data)
                    except EmptyDataError:
                        if skip_empty_files is False:
                            print('Encountered an empty file: ', file.key)
                            return pd.DataFrame([])

        except UnicodeDecodeError:
            for file in bucket.objects.filter(Prefix=key_prefix):
                if 'SUCCESS' not in file.key:
                    data = BytesIO(file.get()['Body'].read())
                    df_list.append(pq.read_pandas(data).to_pandas())
        except ClientError:
            print('File not found on s3')
            return pd.DataFrame([])

    if not df_list:
        print('No matching file found')
        return pd.DataFrame([])
    data = pd.concat(df_list)
    return data


def s3_download_data(s3_filename, prefix=False, output_file=None, sep=',', skip_empty_files=True,
                     first_row_columns=True):
    key_id = config.get_value('aws', 'aws_access_key_id')
    access_key = config.get_value('aws', 'aws_secret_access_key')
    session = boto3.Session(
        aws_access_key_id=key_id,
        aws_secret_access_key=access_key
    )

    s3 = session.resource('s3')

    s3_file_pattern = re.compile(r's3://([^/]+)/?(.*)')

    match = s3_file_pattern.match(s3_filename)
    bucket_name = match.group(1)

    key_prefix = match.group(2)

    if type(sep) == str and len(sep) == 1:

        data = _download_data(key_prefix, s3, bucket_name, prefix, sep, skip_empty_files,
                              first_row_columns)

        if output_file:
            data.to_csv(output_file, sep=sep)

        return data

    else:
        print('Separator must be a 1-character string')


def s3_upload_data(data, bucket, path, sep=','):
    key_id = config.get_value('aws', 'aws_access_key_id')
    access_key = config.get_value('aws', 'aws_secret_access_key')
    session = boto3.Session(
        aws_access_key_id=key_id,
        aws_secret_access_key=access_key
    )

    if type(sep) == str and len(sep) == 1:

        csv_buffer = StringIO()

        if type(data) == pd.core.frame.DataFrame or type(data) == np.ndarray:

            if type(data) == pd.core.frame.DataFrame:
                data.to_csv(csv_buffer, sep=sep)
            elif type(data) == np.ndarray:
                np.savetxt(csv_buffer, data, delimiter=sep, fmt='%s')

            s3 = session.resource('s3')
            data = csv_buffer.getvalue()

            try:
                s3.Bucket(bucket).put_object(Key=path, Body=data)
                print('Success. File saved at s3://{}/{}'.format(bucket, path))
            except TypeError:
                print('Bucket name must be a string')
            except ClientError as e:
                if e.response['Error']['Code'] == 'NoSuchBucket':
                    print('The specified bucket does not exist')
            except ParamValidationError as e:
                print(e)

        else:
            print('Uploaded file must be pandas DataFrame or numpy array and not {}'.format(type(data)))

    else:
        print('Separator must be a 1-character string')



import io
from configparser import NoOptionError
from contextlib import contextmanager

import pandas as pd
from qds_sdk.commands import Command, HiveCommand, PrestoCommand
from qds_sdk.exception import (
    ForbiddenAccess,
    ResourceInvalid,
    ResourceNotFound,
    UnauthorizedAccess,
)
from qds_sdk.qubole import Qubole

import sroka.config.config as config


@contextmanager
def execute_with_handling_errors(func, *args, **kwargs):
    result = None
    try:
        result = func(*args, **kwargs)
    except ResourceInvalid:
        print("Invalid resource")
    except ResourceNotFound:
        print('Resource not found')
    except UnauthorizedAccess:
        print("Invalid credentials were provided")
    except ForbiddenAccess:
        print("Forbidden access")
    except (KeyError, NoOptionError):
        print("No credentials were provided")
    finally:
        yield result


def request_qubole(input_query, query_type='presto', cluster_label=None):
    """Sends SQL query to Qubole and retrieves
    the data as pandas DataFrame.

    :param str input_query: query in chosen language (SQL)
    :param str query_type: query language specification {'presto' (default) or 'hive'}
    :param str cluster_label: Name of the Qubole cluster
    :return:  pandas DataFrame with response data.
    :rtype: pandas.DataFrame
    """
    with execute_with_handling_errors(config.get_value, 'qubole', 'api_token') as api_token:
        if api_token is None:
            return pd.DataFrame([])

    Qubole.configure(api_token=api_token)

    # run query
    if query_type == 'presto':
        with execute_with_handling_errors(PrestoCommand.run, query=input_query, label=cluster_label) as hc:
            if hc is None:
                return pd.DataFrame([])
    elif query_type == 'hive':
        with execute_with_handling_errors(HiveCommand.run, query=input_query, label=cluster_label) as hc:
            if hc is None:
                return pd.DataFrame([])
    else:
        print('Wrong query type')
        return pd.DataFrame([])

    print("Id: %s, Status: %s" % (str(hc.id), hc.status))

    try:
        hc.get_results(fp=open('./temp_qubole_output', 'wb'))
        with open('./temp_qubole_output', 'rb') as f:
            data = f.read()
        return qubole_output_to_df(data)

    except Exception as e:
        print(e)
        print("Oops!  There was a problem.  Try again...")
        return pd.DataFrame([])


def qubole_output_to_df(output):
    """Transforms Qubole output to pandas DataFrame

    :param bytes output: data returned by Qubole
    :return:  pandas DataFrame with response data.
    :rtype: pandas.DataFrame
    """
    lines = output.decode('utf-8').strip().split('\r\n')
    return pd.DataFrame([line.split('\t') for line in lines])


def done_qubole(query_id):
    """Sends query_id to Qubole and retrieves
    the data as pandas DataFrame.

    :param int query_id: query_id ready in Qubole
    :return:  pandas DataFrame with response data.
    :rtype: pandas.DataFrame
    """
    with execute_with_handling_errors(config.get_value, 'qubole', 'api_token') as api_token:
        if api_token is None:
            return pd.DataFrame([])

    Qubole.configure(api_token=api_token)

    with execute_with_handling_errors(Command().find, id=query_id) as res:
        if res is None:
            return pd.DataFrame([])

    print("Id: %s, Status: %s" % (str(res.id), res.status))

    try:
        response_buffer = io.BytesIO()
        res.get_results(response_buffer)
        return qubole_output_to_df(response_buffer.getvalue())

    except Exception as e:
        print(e)
        print("Oops!  There was a problem.  Try again...")
        return pd.DataFrame([])



def validate_input_dict(moat_dict):
    """
    Function that checks if  query dictionary for MOAT API is correct.
    Returns True if it's correct.

    Args:
        moat_dict (dict)

    Returns:
        bool
    """
    for column in ['start', 'end', 'columns']:
        if column not in moat_dict.keys():
            print('{} is not defined'.format(column))
            return False

    if type(moat_dict['columns']) != list or moat_dict['columns'] == []:
        print('columns are not defined correctly')
        return False

    return True



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("requirements.txt") as f:
    requirements = f.read().splitlines()

setuptools.setup(
    name="sroka",
    version="0.0.7",
    author="Ad Engineering FANDOM",
    author_email="murbanek@fandom.com",
    description="Package for access GA, GAM, MOAT, Qubole, Athena, S3, Rubicon APIs, BigQuery, MySQL",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Wikia/sroka",
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ),
    install_requires=requirements
)
